// +build linux !darwin

package sender

import (
	"bufio"
	"cant/can/canbus"
	"cant/can/flowroutines"
	"cant/can/flowroutines/canflow"
	"cant/cmd"
	"cant/cmd/run"
	"cant/util/flow"
	"fmt"

	//"github.com/pkg/profile"
	"os"
	"sync"
	"time"

	"github.com/spf13/cobra"
)

var count int
var logwriter *bufio.Writer
var label string

// init adds the seed command as subcommand to the Root
func init() {
	run.Run.AddCommand(sender)
	sender.PersistentFlags().IntVarP(&count, "count", "c", 100000, "do c iterations")
	sender.PersistentFlags().StringVarP(&label, "label", "l", "cant", "meaningful label for log file")
}

type RTT struct{}

// FlowRun ...
func (c *RTT) FlowRun(in <-chan flowroutines.FlowData, out chan<- flowroutines.FlowData, exit chan bool, wg *sync.WaitGroup) {
	defer wg.Done()
	time.Sleep(30 * time.Second)
	rtts := make([]int64, count)
	data := flowroutines.FlowData{CanFrame: canbus.NewFrame(uint32(0x18EEFF87), true, 8, [8]byte{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF})}
	for i := 0; i < count; i++ {
		start := time.Now()
		out <- data
		<-in
		rtts[i] = time.Since(start).Nanoseconds()
	}
	for i := range rtts {
		logwriter.WriteString(fmt.Sprintf("%d\n", rtts[i]))
	}
	exit <- true
}

// run command
var sender = &cobra.Command{
	Use:               "sender",
	Short:             "run the sender component",
	PersistentPreRun:  func(cmd *cobra.Command, args []string) {}, // overwrite pre runs
	PersistentPostRun: func(cmd *cobra.Command, args []string) {}, // overwrite post runs
	Run: func(cobraCmd *cobra.Command, args []string) {
		//defer profile.Start(profile.MemProfile).Stop()
		//defer profile.Start(profile.CPUProfile).Stop()
		// defer profile.Start(profile.BlockProfile).Stop()
		can0, err := canbus.New("can0")
		if err != nil {
			cmd.ExitWithError(err)
		}
		defer can0.Close()
		f, err := os.Create(fmt.Sprintf("%s_%d_iterations_%d.log", label, count, time.Now().UnixNano()))
		if err != nil {
			cmd.ExitWithError(err)
		}
		logwriter = bufio.NewWriter(f)
		reader := canflow.CreateReader(can0)
		writer := canflow.CreateWriter(can0)
		rtt := &RTT{}
		dataflow := flow.New()
		dataflow.Connect(reader, rtt)
		dataflow.Connect(rtt, writer)
		dataflow.Start()
		dataflow.Join()
		logwriter.Flush()
		f.Sync()
		f.Close()
	},
}
